package de.knauf.fhw.swprojekt.xml;

import java.io.File;
import java.net.URL;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Beispielhafter Reader fr Sherd-Diagrammdateien.
 * 
 * Es wird eine Datei "sherd.xml" im Programmverzeichnis geladen und geparst,
 * die Elemente werden ausgegeben.
 * 
 * Die Datei wird gegen das FH-Schema validiert.
 * 
 * @author Wolfgang Knauf
 * 
 */
public class SherdReader
{
  /**DocumentBuilder fr das Einlesen und Parsen von Sherd-Dokumenten,
   * wird im Konstruktor erzeugt.
   * 
   */
  //private DocumentBuilderFactory documentBuilderFactory ;
  private DocumentBuilder documentBuilder;
  
  /**Konstruktor, ldt das Dokument ein.
   * @exception Exception Fehler beim Initialisieren des DocumentBuilder
   */
  public SherdReader() throws Exception
  {
    DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();

    //Das Ding soll Namespaces beachten:
    documentBuilderFactory.setNamespaceAware(true);
    // "setValidating" betrifft NUR DTD. Fr Schema muss es auf "FALSE" stehen!
    documentBuilderFactory.setValidating(false);

    // Schema in DocumentBuilder packen, sonst keine Validierung dagegen!
    URL urlSchema = new URL("http://www.informatik.fh-wiesbaden.de/~knauf/SWTProjekt2009/sherd.xsd");
    Schema schemaSherd = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI).newSchema(
        urlSchema);
    documentBuilderFactory.setSchema(schemaSherd);
    
    this.documentBuilder = documentBuilderFactory.newDocumentBuilder();
    
    //ErrorHandler zufgen (der wirft Exceptions bei jedem Piep!)
    //Ohne ihn wrde das Parsen nicht abgebrochen, sondern es kmen nur Konsolenausgaben.
    this.documentBuilder.setErrorHandler(new SherdErrorHandler() );
  }

  /**Parsen der bergebenen Datei und Ausgabe der Informationen.
   * 
   * @param sFile Aus dieser Datei wird geladen.
   * @exception Exception Fehler beim Parsen
   */
  public void parse (String sFile) throws Exception
  {
    File fileSherd = new File(sFile);
    //Jeder Parsefehler wird vom SherdErrorHandler weitergeworfen.
    Document docSherd = this.documentBuilder.parse(fileSherd);

    //Element "tabellen" holen:
    NodeList listTabellen = docSherd.getElementsByTagName("tabellen");
    //Seeehr unsauber: auf Verdacht auf Element 0 zugreifen, weil es gem Schema genau eines geben muss.
    //Trotzdem unsauber...
    Element elementTabellen = (Element) listTabellen.item(0);
    //ber alle Childs laufen:
    NodeList listTabelle = elementTabellen.getElementsByTagName("tabelle");
    for (int intIndex = 0; intIndex < listTabelle.getLength(); intIndex++)
    {
      Element elementTabelle = (Element) listTabelle.item(intIndex);
      
      //Debugging-Ausgabe:
      System.out.println ("Tabelle " + intIndex + ": Name = " + elementTabelle.getAttribute("tabellenname") +
          ", ID = " + elementTabelle.getAttribute("id") + 
          ", x = " + elementTabelle.getAttribute("x") + ", y = " + elementTabelle.getAttribute("y") +
          ", width = " + elementTabelle.getAttribute("width") + ", height = " + elementTabelle.getAttribute("height") );
      
      //Spalten auslesen.
      NodeList nodeListSpalten = elementTabelle.getElementsByTagName("spalte");
      for (int intIndexSpalte = 0; intIndexSpalte < nodeListSpalten.getLength(); intIndexSpalte++)
      {
        Element elementSpalte = (Element) nodeListSpalten.item(intIndexSpalte);
    	  System.out.println ("\tSpalte " + intIndexSpalte + ": Name = " + elementSpalte.getAttribute("spaltenname") +
            ", ID = " + elementSpalte.getAttribute("id") + ", Primary Key = " + elementSpalte.getAttribute("primarykey") +
            ", x = " + elementSpalte.getAttribute("x") + ", y = " + elementSpalte.getAttribute("y") +
            ", width = " + elementSpalte.getAttribute("width") + ", height = " + elementSpalte.getAttribute("height") );
        
      }
    }
    
    //Jetzt ber die Comments laufen:
    NodeList listComments = docSherd.getElementsByTagName("comments");
    //Auch hier: auf Verdacht auf Element 0 zugreifen, weil es gem Schema genau eines geben muss.
    Element elementComments = (Element) listComments.item(0);
    //ber alle Childs laufen:
    NodeList listComment = elementComments.getElementsByTagName("comment");
    for (int intIndex = 0; intIndex < listComment.getLength(); intIndex++)
    {
      Element elementComment = (Element) listComment.item(intIndex);
      
      //Debugging-Ausgabe:
      System.out.println ("Kommentar " + intIndex + ": ID= " + elementComment.getAttribute("id") +
          ", x=" + elementComment.getAttribute("x") + ", y=" + elementComment.getAttribute("y") +
          ", width=" + elementComment.getAttribute("width") + ", height=" + elementComment.getAttribute("height") );
      
      //Textinhalt ausgeben:
      System.out.println ("\tKommentartext: " + elementComment.getTextContent() );
    }
  }
  /**Main-Methode, liest eine Datei "sherd.xml" im Programmverzeichnis ein und gibt
   * ihren Inhalt aus.
   * @param args Kommandozeile, hier bedeutungslos
   */
  public static void main(String[] args)
  {
    try
    {
      SherdReader xmlReader = new SherdReader();
      
      xmlReader.parse ("sherd.xml");
      
      System.out.println("Parsed!");
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }

  }

}
