package de.knauf.fhw.swprojekt.xml;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Erzeugt ein Sherd-XML-Dokument.
 * 
 * @author Wolfgang Knauf
 * 
 */
public class SherdWriter
{
  /**XML-Dokument des Diagramms, wird im Konstruktor erzeugt.
   * 
   */
  private Document docSherd;
  
  /**XML-Element fr alle Tabellen, wird im Konstruktor erzeugt.
   * Ist eine Membervariable, damit das Zufgen von Tabellen spter mglich ist.
   */
  private Element elementTabellen;
  
  /**XML-Element fr alle Relationships, wird im Konstruktor erzeugt.
   * Ist eine Membervariable, damit das Zufgen von Relations spter mglich ist.
   */
  private Element elementRelations;
  
  /**XML-Element fr alle Kommentare, wird im Konstruktor erzeugt.
   * Ist eine Membervariable, damit das Zufgen von Kommentaren spter mglich ist.
   */
  private Element elementComments;
  
  /**Erzeugt einen neuen XML-Writer.
   * @throws Exception Alle mglichen XML-Fehler 
   * 
   */
  public SherdWriter() throws Exception
  {
    DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
    DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();

    this.docSherd = documentBuilder.newDocument();
    
    // Rootknoten mit Namespace erzeugen:
    // <sherd ...
    // xmlns="http://www.informatik.fh-wiesbaden.de/~knauf/SWTProjekt2009/sherd">
    Element nodeSherd = this.docSherd.createElementNS("http://www.informatik.fh-wiesbaden.de/~knauf/SWTProjekt2009/sherd",
        "sherd");
    this.docSherd.appendChild(nodeSherd);

    // Quelle:
    // http://bytes.com/groups/xml/468470-how-declare-namespace-prefix-java

    // Erzeugen von gleich zwei Attributen:
    // xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
    // xsi:schemaLocation="http://www.informatik.fh-wiesbaden.de/~knauf/SWTProjekt2009/sherd http://www.informatik.fh-wiesbaden.de/~knauf/SWTProjekt2009/sherd.xsd"
    //Der erste Parameter ist der volle Namespace, der zweite Parameter enthlt Prefix + Name 
    //(wobei der Prefix auf die Namespace-URI verweist), der dritte Parameter ist die eigentliche Schema Location.
    nodeSherd.setAttributeNS(
            "http://www.w3.org/2001/XMLSchema-instance",
            "xsi:schemaLocation",
            "http://www.informatik.fh-wiesbaden.de/~knauf/SWTProjekt2009/sherd http://www.informatik.fh-wiesbaden.de/~knauf/SWTProjekt2009/sherd.xsd");

    // Standard-Attribute:
    nodeSherd.setAttribute("name", "Testdiagramm");
    nodeSherd.setAttribute("version", "0.6");
    nodeSherd.setAttribute("width", "800");
    nodeSherd.setAttribute("height", "600");
    
    //Die zwei Pflichtelemente "tabellen" und "relations" bauen:
    this.elementTabellen = this.docSherd.createElement("tabellen");
    nodeSherd.appendChild(this.elementTabellen);
    this.elementRelations = this.docSherd.createElement("relations");
    nodeSherd.appendChild(this.elementRelations);
    this.elementComments= this.docSherd.createElement("comments");
    nodeSherd.appendChild(this.elementComments);
  }
  
  
  /**Hinzufgen eines "tabelle"-Knotens zum Diagramm.
   * 
   * @param sTabelle Name der hinzuzufgenden Tabelle.
   * @param iX X-Koordinate des Rechtecks mit dem Tabellennamen
   * @param iY Y-Koordinate des Rechtecks mit dem Tabellennamen
   * @param iWidth Breite des Rechtecks mit dem Tabellennamen
   * @param iHeight Hhe des Rechtecks mit dem Tabellennamen
   * @param iId ID der Tabelle
   * @return Erzeugter Knoten
   */
  public Element addTabelleNode(String sTabelle, int iX, int iY, int iWidth, int iHeight, int iId)
  {
    Element elementTabelle = this.docSherd.createElement("tabelle");
    elementTabelle.setAttribute("x", Integer.toString(iX) );
    elementTabelle.setAttribute("y", Integer.toString(iY) );
    elementTabelle.setAttribute("width", Integer.toString(iWidth) );
    elementTabelle.setAttribute("height", Integer.toString(iHeight) );
    elementTabelle.setAttribute("id", Integer.toString(iId) );
    elementTabelle.setAttribute("tabellenname", sTabelle );
    
    this.elementTabellen.appendChild(elementTabelle);
    
    return elementTabelle;
  }
  
  /**Hinzufgen eines "spalte"-Knotens zum bergebenen Tabelle-Knoten oder zum Relationship-Knoten!
   * 
   * @param elementTabelleOrRelation Zu dieser Tabelle oder Relationship wird die Spalte zugefgt. 
   * @param sSpalte Name der hinzuzufgenden Spalte.
   * @param bPrimaryKey TRUE: Primary-Key-Spalte, FALSE: regulre Spalte
   * @param iX X-Koordinate des Rechtecks mit dem Spaltennamen
   * @param iY Y-Koordinate des Rechtecks mit dem Spaltennamen
   * @param iWidth Breite des Rechtecks mit dem Spaltennamen
   * @param iHeight Hhe des Rechtecks mit dem Spaltennamen
   * @param iId ID der Spalte
   * @param sDatentyp Datentyp
   */
  public void addSpalteNode(Element elementTabelleOrRelation, String sSpalte, boolean bPrimaryKey, int iX, int iY, int iWidth, int iHeight, int iId,
      String sDatentyp)
  {
    Element elementSpalte = this.docSherd.createElement("spalte");
    elementSpalte.setAttribute("primarykey", Boolean.toString(bPrimaryKey) );
    elementSpalte.setAttribute("x", Integer.toString(iX) );
    elementSpalte.setAttribute("y", Integer.toString(iY) );
    elementSpalte.setAttribute("width", Integer.toString(iWidth) );
    elementSpalte.setAttribute("height", Integer.toString(iHeight) );
    elementSpalte.setAttribute("id", Integer.toString(iId) );
    elementSpalte.setAttribute("spaltenname", sSpalte);
    elementSpalte.setAttribute("datentyp", sDatentyp);
    
    elementTabelleOrRelation.appendChild(elementSpalte);
  }
  
  /**Hinzufgen eines "comment"-Knotens zum Diagramm.
   * 
   * @param iX X-Koordinate des Rechtecks mit dem Kommentar
   * @param iY Y-Koordinate des Rechtecks mit dem Kommentar
   * @param iWidth Breite des Rechtecks mit dem Kommentar
   * @param iHeight Hhe des Rechtecks mit dem Kommentar
   * @param iId ID des Kommentars
   * @param sComment Kommentartext
   * @return Erzeugter Knoten
   */
  public Element addCommentNode(int iX, int iY, int iWidth, int iHeight, int iId, String sComment)
  {
    Element elementKommentar = this.docSherd.createElement("comment");
    elementKommentar.setAttribute("x", Integer.toString(iX) );
    elementKommentar.setAttribute("y", Integer.toString(iY) );
    elementKommentar.setAttribute("width", Integer.toString(iWidth) );
    elementKommentar.setAttribute("height", Integer.toString(iHeight) );
    elementKommentar.setAttribute("id", Integer.toString(iId) );
    
    //Text als "CDATA" rausschreiben:
    //Markup-Ersetzung ist hier nicht ntig, weil CDATA-Element.
    CDATASection commentText = this.docSherd.createCDATASection(sComment);
    elementKommentar.appendChild(commentText);
    
    //Alternative dazu: "setTextContent" ersetzt Standard-Entities:
    //elementKommentar.setTextContent(sComment);
    
    this.elementComments.appendChild(elementKommentar);
    
    return elementKommentar;
  }
  
  /**Speichern des Dokuments in die bergebene Datei.
   * 
   * @param sFile Zieldatei
   * @exception TransformerException Fehler bei XML-Transformation
   */
  public void save(String sFile) throws TransformerException
  {
    //Quelle:
    //http://www.javazoom.net/services/newsletter/xmlgeneration.html
    // (Beispiel 4 " JAXP + DOM + Serialization ")
    
    // Als XML-Transformation:
    DOMSource domSource = new DOMSource(this.docSherd);
    File fileOutput = new File(sFile);
    StreamResult streamResult = new StreamResult(fileOutput);
    // StreamResult streamResult = new StreamResult(System.out);
    TransformerFactory tf = TransformerFactory.newInstance();

    Transformer serializer = tf.newTransformer();
    serializer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
    serializer.setOutputProperty(OutputKeys.INDENT, "yes");
    serializer.transform(domSource, streamResult);
    
    
  }
  /**Main-Methode, erzeugt eine Instanz des XMLWriter, bastelt ein Dokument und speichert
   * es als Datei "sherd.xml" im Programmverzeichnis.
   * @param args Kommandozeile, hier bedeutungslos
   */
  public static void main(String[] args)
  {
    try
    {
      //Erzeugen des Dokuments:
      SherdWriter writer = new SherdWriter();
      
      //Ein paar Tabellen:
      Element elementTabelle = writer.addTabelleNode("TABELLE1", 10, 11, 20, 21, 99);
      //Spalten:
      writer.addSpalteNode(elementTabelle, "SPALTE1_PK", true, 20, 30, 20, 21, 199, "varchar");
      writer.addSpalteNode(elementTabelle, "SPALTE2", false, 40, 50, 20, 21, 200, "int");
      
      //Und Tabelle ohne Spalten:
      elementTabelle = writer.addTabelleNode("TABELLE2", 100, 101, 20, 21, 100);
      
      //Kommentar:
      elementTabelle = writer.addCommentNode(100, 101, 20, 21, 500, "Kommentar 1");
      elementTabelle = writer.addCommentNode(110, 121, 20, 21, 501, "Kommentar 2 mit Markup wie '<', '>', '&'");
      
      //Speichern:
      writer.save("sherd.xml");
      
      System.out.println("Fertisch");
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }
}
